// Copyright (c) 2014-2021 NetEase, Inc.
// All right reserved.

part of room_kit;

/// 媒体服务
/// TODO接口需要拆分，设置进来
class NERoomMediaService
    with
        _RoomAloggerMixin,
        NERtcStatsEventCallback,
        NERtcDeviceEventCallback,
        NERtcVideoRendererEventListener {

  factory NERoomMediaService() => _getInstance();

  static NERoomMediaService get instance => _getInstance();

  static NERoomMediaService? _instance;

  static NERoomMediaService _getInstance() {
    _instance ??= NERoomMediaService._internal();
    return _instance!;
  }

  late  NERtcEngine _engine;

  late var _isInit = false;

  MediaType _currentMediaType = MediaType.none;
  ///engine
  NERtcEngine get engine => _engine;

  late Set<NERtcStatsEventListener> _rtcStatsEventListeners;
  late Set<NERtcDeviceEventCallback> _rtcDeviceEventListeners;

  NERoomMediaService._internal() {
    _engine = NERtcEngine();
  }

  Future<int> initRtcEngine({
    required NERtcChannelEventCallback rtcChannelEventCallback,
    bool attendeeRecordOn = false,
    required MediaType mediaType,
    required String nrtcAppKey,
  }) async {
    if (_isInit) {
      if (_currentMediaType == mediaType) {
        return NERtcErrorCode.ok;
      } else {
        if (_currentMediaType == MediaType.inRoom) return NERtcErrorCode.ok;
        await engine.release();
        _isInit = !_isInit;
      }
    }

    var options = getOption(attendeeRecordOn);
    var result = await engine.create(
      appKey:nrtcAppKey,
      channelEventCallback: rtcChannelEventCallback,
      options: options,
    );
    _isInit = result == NERtcErrorCode.ok;
    _currentMediaType = mediaType;
    return result;
  }

  /// join channel
  Future<int> joinChannel(JoinRoomInfo joinRoomInfo) async {
    var result = await engine.joinChannel(joinRoomInfo.avRoomCheckSum,
        joinRoomInfo.avRoomCName, joinRoomInfo.avRoomUid);
    commonLogger.i('joinChannel result=$result');
    return result;
  }

  Future leaveChannel() async {
    final result = await engine.leaveChannel();
    commonLogger.i('leaveChannel result=$result');
  }

  /// [NERtcAudioDevice]
  @override
  void onAudioDeviceChanged(int selected) {
    commonLogger.i('onAudioDeviceChanged $selected');
    _rtcDeviceEventListeners.forEach((element) {
      element.onAudioDeviceChanged(selected);
    });
  }

  @override
  void onAudioDeviceStateChange(int deviceType, int deviceState) {
    commonLogger.i('onAudioDeviceStateChange: $deviceType $deviceState');
    _rtcDeviceEventListeners.forEach((element) {
      element.onAudioDeviceStateChange(deviceType, deviceState);
    });
  }

  @override
  void onFrameResolutionChanged(int uid, int width, int height, int rotation) {
    // TODO: implement onFrameResolutionChanged
  }

  @override
  void onRemoteVideoStats(List<NERtcVideoRecvStats> statsList) {
    _rtcStatsEventListeners.forEach((element) {
      element.onRemoteVideoStats(statsList);
    });
  }

  @override
  void onLocalAudioStats(NERtcAudioSendStats stats) {
    _rtcStatsEventListeners.forEach((element) {
      element.onLocalAudioStats(stats);
    });
  }

  @override
  void onRemoteAudioStats(List<NERtcAudioRecvStats> statsList) {
    _rtcStatsEventListeners.forEach((element) {
      element.onRemoteAudioStats(statsList);
    });
  }

  @override
  void onRtcStats(NERtcStats stats) {
    _rtcStatsEventListeners.forEach((element) {
      element.onRtcStats(stats);
    });
  }

  @override
  void onLocalVideoStats(NERtcVideoSendStats stats) {
    _rtcStatsEventListeners.forEach((element) {
      element.onLocalVideoStats(stats);
    });
  }

  @override
  void onNetworkQuality(List<NERtcNetworkQualityInfo> statsList) {
    _rtcStatsEventListeners.forEach((element) {
      element.onNetworkQuality(statsList);
    });
  }

  /// 会议录制模式，0：混合与单人，1：混合，2：单人，但只在有值的时候，才会由服务器抄送给应用的回调接口
  NERtcServerRecordMode get meetingRecordMode {
    var mode = SettingsRepository().meetingRecordMode;
    if (mode >= NERtcServerRecordMode.values.length || mode < 0) {
      mode = 0;
    }
    return NERtcServerRecordMode.values[mode];
  }

  NERtcOptions getOption(bool attendeeRecordOn) {
    return NERtcOptions(
        logLevel: NERtcLogLevel.info,
        logDir: NERoomLogService()._rtcSDKPath,
        audioAutoSubscribe: false,
        videoEncodeMode: useHardwareCodec(),
        videoDecodeMode: useHardwareCodec(),
        serverRecordAudio: SettingsRepository().meetingRecordAudioEnable &&
            attendeeRecordOn &&
            SettingsRepository.isMeetingCloudRecordSupported(),
        serverRecordVideo: SettingsRepository().meetingRecordVideoEnable &&
            attendeeRecordOn &&
            SettingsRepository.isMeetingCloudRecordSupported(),
        serverRecordMode: meetingRecordMode,
        publishSelfStream: true,
        videoCaptureObserverEnabled: true,
        /// IOS 平台美颜回调
        videoSendMode: NERtcVideoSendMode.none,
        serverRecordSpeaker: true);
  }

  @override
  void onFirstFrameRendered(int uid) {
    // TODO: implement onFirstFrameRendered
  }

  @override
  void onVideoDeviceStageChange(int deviceState) {
    commonLogger.i('onVideoDeviceStageChange: $deviceState');
    _rtcDeviceEventListeners.forEach((element) {
      element.onVideoDeviceStageChange(deviceState);
    });
  }

  Future<void> setConfig(
    Set<NERtcStatsEventListener> rtcStatsEventListeners,
    Set<NERtcDeviceEventCallback> rtcDeviceEventListeners,
  ) async {
    await engine.setAudioProfile(
        NERtcAudioProfile.profileDefault, NERtcAudioScenario.scenarioSpeech);
    var config = NERtcVideoConfig()..videoProfile = NERtcVideoProfile.hd720p;
    await engine.setLocalVideoConfig(config);

    _rtcStatsEventListeners = rtcStatsEventListeners;
    ///当用户设置了监听器，则进行rtc监听
    if (_rtcStatsEventListeners.isNotEmpty) {
      await engine.setStatsEventCallback(this);
    }

    _rtcDeviceEventListeners = rtcDeviceEventListeners;
    await engine.deviceManager.setEventCallback(this);
  }

  NERtcMediaCodecMode useHardwareCodec() {
    return defaultTargetPlatform == TargetPlatform.iOS
        ? NERtcMediaCodecMode.hardware
        : NERtcMediaCodecMode.software;
  }

  ///美颜预览配置
  Future<void> defaultConfig() async {
    var config = NERtcVideoConfig();
    config.videoProfile = NERtcVideoProfile.hd720p;
    config.frameRate = NERtcVideoFrameRate.fps_30;
    config.degradationPrefer = NERtcDegradationPreference.degradationDefault;
    config.frontCamera = true;
    config.videoCropMode = NERtcVideoCropMode.cropDefault;
    var result = await _engine.setLocalVideoConfig(config);
    print('setLocalVideoConfig: $result');
    result = await _engine.enableLocalVideo(true);
    print('enableLocalVideo: $result');
    result = await _engine.startVideoPreview();
    print('startVideoPreview: $result');
  }

  Future<void> release(MediaType mediaType) async {
    if (!_isInit || mediaType != _currentMediaType) return;
    _isInit = false;
    _currentMediaType = MediaType.none;
    await _engine.leaveChannel();
    await _engine.enableLocalVideo(false);
    await _engine.enableLocalAudio(false);
    await _engine.stopVideoPreview();
    await _engine.release();
  }
}
enum MediaType { none,preRoom, inRoom }



